import 'package:flutter/material.dart';

/// Coordinates [SfRangeSelector] with,
/// - It's child.
/// - A widget that listened [RangeController].
///
/// The [start] represents the currently selected value of the range selector.
/// The left thumb of the range selector was drawn corresponding to this value.
///
/// The [end] represents the currently selected value of the range selector.
/// The right thumb of the range selector was drawn corresponding to this value.
///
/// [start] and [end] be either [double] or [DateTime].
///
/// {@tool snippet}
///
/// This example demonstrates how to use [RangeController] with
/// [SfRangeSelector] and its child.
///
/// ```dart
/// class RangeSelectorPage extends StatefulWidget {
///   @override
///   _RangeSelectorPageState createState() => _RangeSelectorPageState();
/// }
///
/// class _RangeSelectorPageState extends State<RangeSelectorPage> {
///   SfRangeValues _rangeValues = SfRangeValues(0, 1);
///
///   RangeController _rangeController;
///
///   @override
///   void initState() {
///     super.initState();
///     _rangeController = RangeController(
///       start: _rangeValues.start,
///       end: _rangeValues.end);
///   }
///
///  @override
///  void dispose() {
///    _rangeController.dispose();
///    super.dispose();
///  }
///
///   @override
///   Widget build(BuildContext context) {
///     return Scaffold(
///       body: Column(
///        mainAxisAlignment: MainAxisAlignment.center,
///        children: <Widget>[
///           SfRangeSelector(
///             min: 0,
///             max: 5,
///             controller: _rangeController,
///             child: Container(
///               child: SfCartesianChart(
///                 primaryXAxis: NumericAxis(
///                   controllor: _rangeController
///                 ),
///                 primaryYAxis: NumericAxis(),
///                 series: ...,
///               ),
///             ),
///           ),
///         ],
///       ),
///     );
///   }
/// }
/// ```
///
/// {@end-tool}
///
/// {@tool snippet}
///
/// This example demonstrates how to use [RangeController]
/// with [SfRangeSelector] and a widget that listened [RangeController].
///
/// ```dart
/// class RangeSelectorPage extends StatefulWidget {
///   @override
///   _RangeSelectorPageState createState() => _RangeSelectorPageState();
/// }
///
/// class _RangeSelectorPageState extends State<RangeSelectorPage> {
///   SfRangeValues _rangeValues = SfRangeValues(0, 1);
///
///   RangeController _rangeController;
///
///   @override
///   void initState() {
///     super.initState();
///     _rangeController = RangeController(
///       start: _rangeValues.start,
///       end: _rangeValues.end);
///   }
///
///  @override
///  void dispose() {
///    _rangeController.dispose();
///    super.dispose();
///  }
///
///   @override
///   Widget build(BuildContext context) {
///     return Scaffold(
///       body: Column(
///        mainAxisAlignment: MainAxisAlignment.center,
///        children: <Widget>[
///           Container(
///               child: SfCartesianChart(
///                 primaryXAxis: NumericAxis(
///                   controllor: _rangeController
///                 ),
///                 primaryYAxis: NumericAxis(),
///                 series: ...,
///             ),
///           ),
///           SfRangeSelector(
///             min: 0,
///             max: 5,
///             controller: _rangeController,
///             child: ...,
///           ),
///         ],
///       ),
///     );
///   }
/// }
/// ```
///
/// {@end-tool}
class RangeController extends ChangeNotifier {
  /// Creates a new instance of [RangeController].
  ///
  /// The [start] represents the currently selected value of the range selector.
  /// The left thumb of the range selector was drawn corresponding to this value.
  ///
  /// The [end] represents the currently selected value of the range selector.
  /// The right thumb of the range selector was drawn corresponding to this value.
  ///
  /// [start] and [end]
  RangeController({@required dynamic start, @required dynamic end})
      : assert(start != null),
        assert(end != null),
        _previousStart = start,
        _previousEnd = end,
        _start = start,
        _end = end;

  /// The current selected start value.
  ///
  /// It be either [double] or [DateTime].
  dynamic get start => _start;
  dynamic _start;
  set start(dynamic value) {
    assert(value != null);
    _previousStart = start;
    if (_start == value) {
      return;
    }
    _start = value;
    if (value != null) {
      notifyListeners();
    }
  }

  /// The current selected end value.
  ///
  /// It be either [double] or [DateTime].
  dynamic get end => _end;
  dynamic _end;
  set end(dynamic value) {
    assert(value != null);
    _previousEnd = end;
    if (_end == value) {
      return;
    }
    _end = value;
    if (value != null) {
      notifyListeners();
    }
  }

  /// The previously selected start value.
  ///
  /// It be either [double] or [DateTime].
  dynamic get previousStart => _previousStart;
  dynamic _previousStart;

  /// The previously selected end value.
  ///
  /// It be either [double] or [DateTime].
  dynamic get previousEnd => _previousEnd;
  dynamic _previousEnd;
}
